﻿using System.ServiceModel;
using DotNetSamples.Common;
using NUnit.Framework;

namespace DotNetSamples.Tests
{
    [TestFixture]
    public class DotNetTests
    {
        [Test]
        public void TestCanCallDotNetServiceWithTokenFromUserNameAuth()
        {
            // Hent security token
            var securityToken = WsTrustClient.RequestSecurityTokenWithUserName(
                Constants.StsAddressUserName,
                Constants.StsCertificate,
                Constants.DotNetServiceAddress,
                Constants.DmpUserName,
                Constants.DmpPassword);

            // Verificer, at token'et returneres
            Assert.That(securityToken, Is.Not.Null);

            // Kald service
            var result = TestCommon.TestCanCallServiceWithIssuedToken(securityToken);

            Assert.That(result, Is.Not.Null);
            Assert.That(result.Text, Is.StringMatching("Hello .NET!"));
            Assert.That(result.ClaimsCount, Is.GreaterThan(0));
        }

        [Test]
        public void TestCanCallServicewithTokenAndThatServiceHasBootstrapToken()
        {
            // Hent token
            var securityToken = WsTrustClient.RequestSecurityTokenWithUserName(
                Constants.StsAddressUserName,
                Constants.StsCertificate,
                Constants.DotNetServiceAddress,
                Constants.DmpUserName,
                Constants.DmpPassword);

            // Verificer token'et er der
            Assert.That(securityToken, Is.Not.Null);

            // Kald service
            TestCommon.TestCanCallServiceWithIssuedTokenAndVerifyBootstrap(securityToken);
        }

        [Test]
        public void TestCanCallServiceWithTokenFromX509Auth()
        {
            // Hent security token
            var securityToken = WsTrustClient.RequestSecurityTokenWithX509(
                Constants.StsAddressCertificate,
                Constants.StsCertificate,
                Constants.DotNetServiceAddress,
                Constants.GetClientCertificateTest());

            // Verificer, at token'et returneres
            Assert.That(securityToken, Is.Not.Null);

            // Kald service
            var result = TestCommon.TestCanCallServiceWithIssuedToken(securityToken);

            Assert.That(result, Is.Not.Null);
            Assert.That(result.Text, Is.StringMatching("Hello .NET!"));
            Assert.That(result.ClaimsCount, Is.GreaterThan(0));
        }

        [Test]
        public void TestCanCallServicewithTokenAndThatServiceHasBootstrapTokenAndThatServiceCanActAsToJavaService()
        {
            // Hent token
            var securityToken = WsTrustClient.RequestSecurityTokenWithUserName(
                Constants.StsAddressUserName,
                Constants.StsCertificate,
                Constants.DotNetServiceAddress,
                Constants.DmpUserName,
                Constants.DmpPassword);

            // Verificer token'et er der
            Assert.That(securityToken, Is.Not.Null);

            // Kald service
            var javaResult = TestCommon.TestCanCallServiceWithIssuedTokenAndVerifyBootstrapAndCallJavaServiceWithActAs(
                securityToken);

            Assert.That(javaResult, Is.Not.Null);
            Assert.That(javaResult.Text, Is.StringMatching("Hello Java!"));
        }
    }
}
