﻿using System;
using System.ServiceModel;
using DotNetSamples.Common;
using NUnit.Framework;

namespace DotNetSamples.Tests
{
    public class GetTokenTests
    {
        [Test]
        public void TestCanReceiveTokenWithUserNameAuthentication()
        {
            // Hent security token
            var securityToken = WsTrustClient.RequestSecurityTokenWithUserName(
                Constants.StsAddressUserName,
                Constants.StsCertificate,
                Constants.DotNetServiceAddress,
                Constants.DmpUserName,
                Constants.DmpPassword);

            // Verificer, at token'et returneres
            Assert.IsNotNull(securityToken);
        }

        [Test]
        public void TestCanReceiveTokenWithX509Authentication()
        {
            // Hent security token
            var securityToken = WsTrustClient.RequestSecurityTokenWithX509(
                Constants.StsAddressCertificate,
                Constants.StsCertificate,
                Constants.DotNetServiceAddress,
                Constants.GetClientCertificateTest());

            // Verificer, at token'et returneres
            Assert.IsNotNull(securityToken);
        }

        [Test]
        [Ignore("Kræver at der indtastes adresse på lokal IdP")]
        public void TestCanGetLocalIdPToken()
        {
            // Indsæt adresse på lokal AD FS 2.0
            // Bemærk, dette skal erstattes med din lokale AD FS 2.0 server
            // og denne AD FS 2.0 server skal have end pointet 
            // "adfs/services/trust/13/windows" enablet
            var stsAddress = new EndpointAddress(
                new Uri("http://adfs.c0d3.dk/adfs/services/trust/13/windows"),
                EndpointIdentity.CreateDnsIdentity("http://adfs.c0d3.dk/adfs/services/trust"));

            // Det end point på STS'en der modtager token'et fra den lokale IdP
            var dmpIssuedTokenAddress = Constants.StsAddressIssuedToken;

            // Hent lokalt security token vha. Windows Authentication, 
            // da vi er logget på domænet med den lokale IdP
            var localToken = WsTrustClient.RequestSecurityTokenWithWindowsAuth(
                stsAddress,
                dmpIssuedTokenAddress);

            Assert.That(localToken, Is.Not.Null);
        }

        [Test]
        [Ignore("Kræver at der indtastes adresse på lokal IdP")]
        public void TestCanExchangeLocalIdPTokenForToken()
        {
            // Indsæt adresse på lokal AD FS 2.0 
            var stsAddress = new EndpointAddress(
                new Uri("http://adfs.c0d3.dk/adfs/services/trust/13/windows"),
                EndpointIdentity.CreateDnsIdentity("http://adfs.c0d3.dk/adfs/services/trust"));

            // Det end point på STS'en der modtager token'et fra den lokale IdP
            var dmpIssuedTokenAddress = Constants.StsAddressIssuedToken;

            // Hent lokalt security token vha. Windows Authentication, 
            // da vi er logget på domænet med den lokale IdP
            var localToken = WsTrustClient.RequestSecurityTokenWithWindowsAuth(
                stsAddress,
                dmpIssuedTokenAddress);

            // Udveksl token'et til et DMP token, som efterfølgende kan bruges til et servicekald
            var securityToken = WsTrustClient.RequestSecurityTokenWithIssuedTokenAuth(
                dmpIssuedTokenAddress,
                Constants.DotNetServiceAddress,
                stsAddress,
                localToken,
                Constants.StsCertificate);

            Assert.That(securityToken, Is.Not.Null);
        }
    }
}
