﻿using System;
using System.IdentityModel.Tokens;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Security;
using DotNetSamples.Common;
using DotNetSamples.WcfClient.JupiterWriteNs;

namespace DotNetSamples.WcfClient
{
    class Program
    {
        //Dette eksempel kalder JupiterWrite Java Service ved brug af Test miljøets STS
        static void Main()
        {
            // For direkte login mod DMP
            var tempBinding = CreateServiceBindingDirectLogin();
            
            // Deaktiver secure conversation hvis der kaldes en Java service
            var serviceBinding = WsTrustClient.CreateWsFederationBindingWithoutSecureConversation(tempBinding);

            var cf = new ChannelFactory<JupiterWrite>(serviceBinding, Constants.JavaServiceAddress);
            var credentials = cf.Credentials;

            if (credentials == null)
                throw new InvalidOperationException("ClientCredentials not present. Client is not correctly configured.");

            // Til DMP login
            credentials.UserName.UserName = Constants.DmpUserName;
            credentials.UserName.Password = Constants.DmpPassword;

            // Sæt public keys på hhv. AD FS 2.0 channel og Service channel da negotiation er slået fra
            credentials.ServiceCertificate.ScopedCertificates.Add(Constants.StsAddressUserName.Uri, Constants.StsCertificate);
            credentials.ServiceCertificate.ScopedCertificates.Add(Constants.JavaServiceAddress.Uri, Constants.ServiceCertificate);

            // Deaktivering af certifikatvalidering på servicecertifikatet. Må IKKE være aktiveret i et produktionsmiljø, 
            // da der her skal anvendes offentligt udstedte certifikater, som kan valideres
            credentials.ServiceCertificate.Authentication.CertificateValidationMode = X509CertificateValidationMode.None;
            credentials.ServiceCertificate.Authentication.RevocationMode = X509RevocationMode.NoCheck;

            var client = cf.CreateChannel();

            try
            {
                const string sql = "SELECT * FROM PCJUPITERXL.DRWPLANT WHERE ROWNUM = 25";

                // Kald servicen
                var result = client.selectPw(new selectPwRequest(sql));

                // Udskriv svar
                Console.WriteLine(result);

                ((ICommunicationObject)client).Close();
            }
            catch(Exception ex)
            {
                Console.WriteLine(ex.ToString());
                ((ICommunicationObject)client).Abort();
            }
        }

        private static WS2007FederationHttpBinding CreateServiceBindingDirectLogin()
        {
            // Lav binding til STS'en
            var stsBinding = new WS2007HttpBinding(SecurityMode.Message, false);
            stsBinding.Security.Message.ClientCredentialType = MessageCredentialType.UserName;
            stsBinding.Security.Message.EstablishSecurityContext = false;
            stsBinding.Security.Message.NegotiateServiceCredential = false;

            // Lav binding til servicen
            var serviceBinding = new WS2007FederationHttpBinding(WSFederationHttpSecurityMode.Message, false);
            serviceBinding.Security.Message.AlgorithmSuite = SecurityAlgorithmSuite.Basic256;
            serviceBinding.Security.Message.IssuedKeyType = SecurityKeyType.SymmetricKey;
            serviceBinding.Security.Message.IssuedTokenType = "http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV1.1";
            serviceBinding.Security.Message.NegotiateServiceCredential = false;
            serviceBinding.Security.Message.IssuerAddress = Constants.StsAddressUserName;
            serviceBinding.Security.Message.IssuerBinding = stsBinding;
            serviceBinding.Security.Message.IssuerMetadataAddress = new EndpointAddress("https://log-in.test.miljoeportal.dk/runtime/services/trust/mex");

            return serviceBinding;
        }
    }
}
