﻿using System;
using System.IdentityModel.Tokens;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Security;
using DotNetSamples.Common;
using DotNetSamples.Common.Contract;
using GAD.LoggingToolBox;

namespace TestConsole
{
    class Program
    {
        private static void Main()
        {
            TetsGetTokens();
            TestCallService();
        }

        private static void TetsGetTokens()
        {
            Console.Write("Enter user name: ");
            var userName = Console.ReadLine();

            Console.Write(Environment.NewLine + "Enter password: ");
            var password = Console.ReadLine();

            var totalTimer = new HiPerfTimer();
            totalTimer.Start();

            var tokenTimer = new HiPerfTimer();
            double min = 0;
            double max = 0;
            for (var i = 0; i < Iterations; i++)
            {
                tokenTimer.Start();
                var token = GetToken(userName, password);

                var tokenTime = tokenTimer.Stop();

                if (tokenTime < min || min == 0)
                    min = tokenTime;

                if (tokenTime > max || max == 0)
                    max = tokenTime;
            }

            var totalTime = totalTimer.Stop();

            Console.WriteLine("Total time for: {0} tokens was: {1}. Average per token: {2}. Min: {3}. Max: {4}",
                              Iterations,
                              totalTime,
                              totalTime/Iterations,
                              min,
                              max);
        }

        private static void TestCallService()
        {
            Console.Write("Enter user name: ");
            var userName = Console.ReadLine();

            Console.Write(Environment.NewLine + "Enter password: ");
            var password = Console.ReadLine();

            var serviceToken = GetToken(userName, password);

            // Lav binding til STS'en
            var stsBinding = WsTrustClient.GetStsBinding();

            // Lav binding til servicen
            var serviceBinding = WsTrustClient.GetServiceBinding(stsBinding, Constants.StsAddressUserName, true);

            // Lav channel til servicen
            var channelFactory = new ChannelFactory<IServiceContract>(serviceBinding, Constants.DotNetServiceAddress);

            if (channelFactory.Credentials == null)
                throw new ApplicationException("ChannelFactory must have credentials");

            // Deaktiver service cert validering
            channelFactory.Credentials.ServiceCertificate.Authentication.CertificateValidationMode =
                X509CertificateValidationMode.None;
            channelFactory.Credentials.ServiceCertificate.Authentication.RevocationMode = X509RevocationMode.NoCheck;

            // Påhæft token til kald til service
            var channel = channelFactory.CreateChannelWithIssuedToken(serviceToken);

            var totalTimer = new HiPerfTimer();
            totalTimer.Start();

            var serviceTimer = new HiPerfTimer();
            double min = 0;
            double max = 0;
            try
            {
                for (var i = 0; i < Iterations; i++)
                {
                    serviceTimer.Start();
                    // Lav query
                    var query = new HelloWorldQuery
                                    {
                                        Text = "Jesper Hvid"
                                    };


                    // Kald service
                    channel.HelloWorld(query);

                    var serviceTime = serviceTimer.Stop();

                    if (serviceTime < min || min == 0)
                        min = serviceTime;

                    if (serviceTime > max || max == 0)
                        max = serviceTime;
                }
            }
            finally
            {
                WsTrustClient.CloseChannel(channel);
            }
            
            var totalTime = totalTimer.Stop();

            Console.WriteLine("Total time for: {0} service calls was: {1}. Average per service call: {2}. Min: {3}. Max: {4}",
                Iterations,
                totalTime,
                totalTime / Iterations,
                min,
                max);
        }

        private static SecurityToken GetToken(string userName, string password)
        {
            var token = WsTrustClient.RequestSecurityTokenWithUserName(
                Constants.StsAddressUserName,
                Constants.StsCertificate,
                Constants.DotNetServiceAddress,
                userName,
                password);
            return token;
        }

        private const int Iterations = 10;

    }
}
